from ..part.plate import Reflector


def reflector_b():
    """Reflector B."""
    return Reflector('YRUHQSLDPXNGOKMIEBFZCWVJAT', name='Reflector B')


def reflector_c():
    """Reflector C."""
    return Reflector('FVPJIAOYEDRZXWGCTKUQSBNMHL', name='Reflector C')


def reflector_b_thin():
    """Reflector B Thin."""
    return Reflector('ENKQAUYWJICOPBLMDXZVFTHRGS', name='Reflector B Thin')


def reflector_c_thin():
    """Reflector C Thin."""
    return Reflector('RDOBJNTKVEHMLFCWZAXGYIPSUQ', name='Reflector C Thin')
